import { logger } from '../../logger';
import { cache } from '../../util/cache/package/decorator';
import type { HttpResponse } from '../../util/http';
import * as hexVersioning from '../../versioning/hex';
import { Datasource } from '../datasource';
import type { GetReleasesConfig, ReleaseResult } from '../types';
import type { HexRelease } from './types';

export class HexDatasource extends Datasource {
  static readonly id = 'hex';

  constructor() {
    super(HexDatasource.id);
  }

  override readonly defaultRegistryUrls = ['https://hex.pm/'];

  override readonly customRegistrySupport = false;

  override readonly defaultVersioning = hexVersioning.id;

  @cache({
    namespace: `datasource-${HexDatasource.id}`,
    key: ({ lookupName }: GetReleasesConfig) => lookupName,
  })
  async getReleases({
    lookupName,
    registryUrl,
  }: GetReleasesConfig): Promise<ReleaseResult | null> {
    // Get dependency name from lookupName.
    // If the dependency is private lookupName contains organization name as following:
    // hexPackageName:organizationName
    // hexPackageName is used to pass it in hex dep url
    // organizationName is used for accessing to private deps
    const hexPackageName = lookupName.split(':')[0];
    const hexUrl = `${registryUrl}api/packages/${hexPackageName}`;

    let response: HttpResponse<HexRelease>;
    try {
      response = await this.http.getJson<HexRelease>(hexUrl);
    } catch (err) {
      this.handleGenericErrors(err);
    }

    const hexRelease: HexRelease = response.body;

    if (!hexRelease) {
      logger.warn({ datasource: 'hex', lookupName }, `Invalid response body`);
      return null;
    }

    const { releases = [], html_url: homepage, meta } = hexRelease;

    if (releases.length === 0) {
      logger.debug(`No versions found for ${hexPackageName} (${hexUrl})`); // prettier-ignore
      return null;
    }

    const result: ReleaseResult = {
      releases: releases.map(({ version, inserted_at }) =>
        inserted_at
          ? {
              version,
              releaseTimestamp: inserted_at,
            }
          : { version }
      ),
    };

    if (homepage) {
      result.homepage = homepage;
    }

    if (meta?.links?.Github) {
      result.sourceUrl = hexRelease.meta.links.Github;
    }

    return result;
  }
}
